#    most of this task is now taken from the standard application

set VERSION "GRT4P"

#    put some default environment variables before loading common 

if {[info exists env(RDOUThdwre)] == 0} then {set env(RDOUThdwre) VME}

send vmecontrol start
send vmecontrol close-frame
after 2000
set RDOUTserverList [inform vmecontrol inquire VMEServerList]

set DAProgram GRT4PExecV5

loadsource  sys.common

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]

lappend actionlist [list "Raw Data Display"  "Dump_Raw_VME_Data"]
lappend actionlist [list "Print Hardware Configuration"  "PrintConfig"]


if {[llength $RDOUTserverList] != [llength $SpectrumServerList]} then {
    set s [first $SpectrumServerList]
    set SpectrumServerList $s
    for {set i 2} {$i <= [llength $RDOUTserverList]} {incr i} {lappend SpectrumServerList $s$i}
}

set frametitle "MIDAS Data Acquisition Experiment Control (GRT4P)"

set VMEPresent 1
set VMEEnable 1

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set TestBuffer ""

set UseGroup 0

set S9418_Base 64
set Options(s9418_GroupBase) $S9418_Base
update_Options


proc create_s9418_spectra {} {
    global SpectrumServer UseGroup RDOUTserver S9418_Base

   set maxmodule [inform vmecontrol inquire s9418count]
   set maxadc 32

   DefineConfig "9418 0 [format "0x%x" $S9418_Base]"

   set z [catch {eg "(register :name '/$RDOUTserver/UseGroup'  ?value)" } UseGroup]

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module s9418#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName s9418#$i.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum s9418#$i.adc$j"
          clear-footer
          error $m
      } else {
          if {$UseGroup == 0} {
              set group [expr $S9418_Base - 1 + $i]
              map_spectrum  "$spectrumname $group $j 0 0"
          } else {
              set group [expr (($i - 1) * 32) + $S9418_Base + $j]
              map_spectrum  "$spectrumname $group 0 0 0"
          }
      }
    }
   }
}

proc create_spectra {} {
    global SpectrumServer SpectrumServerList RDOUTserver RDOUTserverList HistMode

   if {$HistMode == 0} {return}
   
   set-footer "Creating spectra"

   foreach server $SpectrumServerList {
       set SpectrumServer $server

       set z [catch {eg "(spectrum :path '/$SpectrumServer' :range [expr 320 * 64] :name 'Stat' !new)" } m]
       set z [catch {eg "(spectrum :path '/$SpectrumServer' :range [expr 320 * 64] :name 'Rate' !new)" } m]
   }
   map_spectrum "Stat 0 0"

#   for silena 9418

    create_s9418_spectra

#   for the GRP 4 channel module

   set ADCs [list A B C D]

   set maxmodule [inform vmecontrol inquire grt4pcount]
   set maxadc 4

   for {set i 1} {$i <= $maxmodule} {incr i} {
    set VMEserver [inform vmecontrol VMEmodule2server grt4p#$i]
    set RDOUTserver $VMEserver
    set k [lsearch $RDOUTserverList $VMEserver]
    set SpectrumServer [lindex $SpectrumServerList $k]
    
    for {set j 0} {$j < $maxadc} {incr j} {

#    histogram used to display the digital trace
#    need to obtain trace length from the board

      set z [catch {eg "(register :name '/$VMEserver/grt4p#$i.Ch[lindex $ADCs $j].DPP_Reg6'  ?value)" } vl]
      if {$z!=0} then {set vl 512}      ;# use default 
      if {$vl==0} then {set vl 512}     ;# use default

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range [expr $vl - 2] :name 'grt4p#$i.adc$j' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.adc$j"
          clear-footer
          error $m
      } else {
          map_spectrum  "grt4p#$i.adc$j [expr 256 + $j + ($i * 4)] 0 0 0"
      }

#    histograms for the computed energy 

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'grt4p#$i.E[set j]L' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.E[set j]L"
          clear-footer
          error $m
      } else {
          map_spectrum  "grt4p#$i.E[set j]L [expr $j + ($i * 4)] 0 0 0"         ;#  item 0
      }

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'grt4p#$i.E[set j]H' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.E[set j]H"
          clear-footer
          error $m
      } else {          
          map_spectrum  "grt4p#$i.E[set j]H [expr $j + ($i * 4)] 1 0 0"        ;#  item 1
      }
    }
   }
}

proc define_VME_addresses {} {
    global VMEserver RDOUTserver

    set RDOUTserver $VMEserver

#    load hardware configuration into readout software

   ClearConfig

#    first all the ADCs

   set maxmodule [inform vmecontrol inquire grt4pcount]
   for {set i 1} {$i <= $maxmodule} {incr i} {

      set VMEserver [inform vmecontrol VMEmodule2server grt4p#$i]
      set z [catch {eg "(register :name '/$VMEserver/grt4p#$i.ChA.DPP_CSR'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register grt4p#$i.ChA.DPP_CSR"
          clear-footer
          error $m
      }
      DefineConfig "4 $i [string range [first $m] 2 end]"
   }

#    the S9418 VME modules

     define_S9418_config

#    then the acquisition control module

     define_sac_config

     PrintConfig
}

proc Custom_PreSetup {args} {define_VME_addresses}

proc redisplay-vme {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]
       set vl [read-registers xvme260#1.PortA]
       if {$vl != ""} then {
           if {[second [first $vl]] == "0x0000"} {:modify VME :value 1} else {:modify VME :value 0}
           return
       }
       set vl [read-registers sac#1.Enable]
       if {$vl != ""} then {
           :modify VME :value [expr [second [first $vl]] & 1]]
           return
       }
       :modify VME :value -1
}

proc Stop_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       write-registers xvme260#1.PortA 0

       write-registers sac#1.STR 0
}

proc Go_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       write-registers sac#1.MON0 2
       write-registers sac#1.STR 9
       write-registers sac#1.ARM 0

       write-registers xvme260#1.PortA -1
}

proc Reset_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       click-vme 2  ;#  Called during GO - Will cause Soft Reset of all channels to flush FIFOs etc

       write-registers sac#1.EVCLR 0

       write-registers xvme260#1.PortA 0
}


proc make-test-frame {} {

  global actionlist

  :frame [self frame-name].adv :label "System Configuration, Test and Debug Only" :show-footer T :resize F

    :panel ADVP1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 5

        set i 0
        foreach item $actionlist {
            :button A$i :label "[first $item]" :command "[second $item]" :w 24
            incr i
            if {[expr ($i/4)*4] == $i} {:next-row T}
        }

     :next-row T

    :message M011 :label "Data Acquisition System Executive" :bold T
      :next-row T
    :message M012 :label "program status currently not known"

      :next-row T

    :message M021 :label "Data Acquisition Program" :bold T
      :next-row T
    :message M022 :label "program status currently not known"
      :next-row T
       :text DAPGM      :label "Data Acquisition Program" :w 16
       :text DADIR      :label "Data Acquisition Program Directory" :w 44
      :next-row T
       :button DATERM   :label "Terminate Data Acquisition Program"
       :button DALOAD   :label "Load Data Acquisition Program"
       :hskip 100
       :button DAKILL   :label "Kill Data Acquisition Program"

      :next-row T

    :message M1 :label "program options" :bold T
     :next-row T
      :layout V

       :checkbox SCANTABLE  :label "Enable Scanning Table Controls" :valuex 32c :command do_scantable
       :checkbox USETIME    :label "Use Time Stamp control"         :valuex 32c :command do_option
       :checkbox USECOUNT   :label "Use Event Counter control"      :valuex 32c :command do_option
       :checkbox VERBOSE    :label "Enable Verbose mode"            :valuex 32c :command do_option
       :text     TIMERANGE  :label "Time Stamp compare range"       :valuex 32c :command do_option :w 6

    :next-row T
    :message M2 :label "tape server options" :bold T
      :next-row T
      :layout H

       :col-gap 15
       :text TSSERVER      :label "Tape Server" :w 12 :valuex 12c :command do_tsserver
       :text TSPORT        :label "Port"        :w 8  :command do_tsport
       :hskip 30
       :button TSDEFINE    :label "Setup Connection"  :command do_tsdefine
       :button TSUNDEFINE  :label "Cancel Connection" :command do_tsundefine

    :next-row T
       :button UPDATE :command update-test-frame

  :show T

    update-test-frame
}

set scantable 0
set usetime 0
set usecount 0
set verbose 0
set timerange 100

proc update-test-frame {} {
    global scantable usetime usecount verbose timerange
   global OS DAProgram DADirectory LynxOSDirectory LinuxDirectory

   checkinstallation
   if {$OS == "Linux"} then {
       set DADirectory $LinuxDirectory
   } else {
       set DADirectory $LynxOSDirectory
   }

    set ID [read-register Executive.ID]
    if {[llength $ID] == 3} then {
        set PID [read-register Executive.PID]
        :modify M012 :label "Target OS = [first $ID];  Program = [second $ID];  Version = [third $ID]; PID = $PID"
        set PID [read-register EXEC.PID]
    } else {
        :modify M012 :label "program status currently not known"
    }

    set ID [read-register EXEC.ID]
    if {[llength $ID] == 3} then {
        set PID [read-register EXEC.PID]
        :modify M022 :label "Target OS = [first $ID];  Program = [second $ID];  Version = [third $ID]; PID = $PID"
    } else {
        :modify M022 :label "program status currently not known"
    }

    :modify DAPGM :value $DAProgram
    :modify DADIR :value $DADirectory

       :modify TSSERVER    :value [read-register TS_Server]
       :modify TSPORT      :value [read-register TS_Port]

       :modify SCANTABLE  :value $scantable
       :modify USETIME    :value $usetime
       :modify USECOUNT   :value $usecount
       :modify VERBOSE    :value $verbose
       :modify TIMERANGE  :value $timerange
}

proc do_tsserver {w s}  {write-register TS_Server $s}
proc do_tsport   {w v}  {write-register TS_Port $v}
proc do_tsdefine {}     {click-special 3}
proc do_tsundefine {}   {click-special 1}

proc do_scantable {w v} {
    global scantable
 
    set scantable [expr $v*2]
    execute-command 16 "" "$scantable 0 0 0 0 0" 2
}

proc do_option {w v} {
    global usetime usecount verbose timerange

    switch $w {
        VERBOSE   {set verbose $v}
        USETIME   {set usetime $v}
        USECOUNT  {set usecount $v}
        TIMERANGE {set timerange $v}
    }

    execute-command 17 "" "$verbose $usetime $usecount $timerange" 2
}

proc do_clearstats {} {execute-command 7  "Clear Statistics"}

proc enable_scanning {} {execute-command 16 "" "3 1 2 3 4 5" 2}


proc click-histenable {v} {
    global RDOUThdwre RDOUTEnable histEnable
    if {$RDOUTEnable == 0} then {set  histEnable 0} else {set histEnable $v}
    if {$RDOUTEnable != 0} then {write-register $RDOUThdwre.Chistenable [expr $histEnable * 10]}
    redisplay-histenable
}
